EXPORTS
	T1_AAFillOutline
	T1_AAGetLevel
	T1_AAHSetGrayValues
	T1_AANSetGrayValues
	T1_AASetBitsPerPixel
	T1_AASetChar
	T1_AASetGrayValues
	T1_AASetLevel
	T1_AASetSmartLimits
	T1_AASetSmartMode
	T1_AASetString
	T1_AbsolutePath
	T1_AddFont
	T1_AddToFileSearchPath
	T1_CheckEndian
	T1_CloseLib
	T1_ComputeLineParameters
	T1_ConcatGlyphs
	T1_ConcatOutlines
	T1_CopyFont
	T1_CopyGlyph
	T1_DeleteAllSizes
	T1_DeleteEncoding
	T1_DeleteFont
	T1_DeleteSize
	T1_DumpGlyph
	T1_DumpPath
	;T1_DumpPixmap
	T1_ExtendFont
	T1_ExtendHMatrix
	T1_ExtendVMatrix
	T1_FillOutline
	T1_FreeGlyph
	T1_GenerateAFMFallbackInfo
	T1_Get_no_fonts
	T1_GetAfmFileName
	T1_GetAllCharNames
	T1_GetBitmapPad
	T1_GetCharBBox
	T1_GetCharName
	T1_GetCharOutline
	T1_GetCharWidth
	T1_GetEncodingIndex
	T1_GetEncodingScheme
	T1_GetFamilyName
	T1_GetFileSearchPath
	T1_GetFontBBox
	T1_GetFontFileName
	T1_GetFontName
	T1_GetFullName
	T1_GetIsFixedPitch
	T1_GetItalicAngle
	T1_GetKerning
	T1_GetLibIdent
	T1_GetLinePosition
	T1_GetLineThickness
	T1_GetMetricsInfo
	T1_GetMoveOutline
	T1_GetNotice
	T1_GetStringBBox
	T1_GetStringOutline
	T1_GetStringWidth
	T1_GetUnderlinePosition
	T1_GetUnderlineThickness
	T1_GetVersion
	T1_GetWeight
	T1_InitLib
	T1_LoadEncoding
	T1_LoadFont
	T1_ManipulatePath
	T1_MirrorHMatrix
	T1_MirrorVMatrix
	T1_PrintLog
	T1_QueryLigs
	T1_QueryX11Support
	T1_ReencodeFont
	T1_RelativePath
	T1_RotateMatrix
	T1_ScaleOutline
	T1_SetAfmFileName
	T1_SetBitmapPad
	T1_SetChar
	T1_SetDefaultEncoding
	T1_SetDeviceResolutions
	T1_SetFileSearchPath
	T1_SetFontDataBase
	T1_SetLinePosition
	T1_SetLineThickness
	T1_SetLogLevel
	T1_SetRasterFlags
	T1_SetString
	T1_ShearHMatrix
	T1_ShearVMatrix
	T1_SlantFont
	T1_TransformMatrix
	T1_WriteAFMFallbackFile
;	T1_AADoLine
;	T1_AAInit
;	T1_DoLine
;	T1Close
;	T1Decrypt
;	T1eexec
;	T1Fill
;	T1Getc
;	T1io_reset
;	T1lib_parseFile
;	T1Open
;	T1Read
;	T1Ungetc
